@@ -5,7 +5,7 @@ use Carp qw/carp croak confess/;
 
 BEGIN {
   $Math::Prime::Util::ECProjectivePoint::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::ECProjectivePoint::VERSION = '0.42';
+  $Math::Prime::Util::ECProjectivePoint::VERSION = '0.46';
 }
 
 BEGIN {
@@ -142,18 +142,18 @@ sub double {
   return $self;
 }
 
-sub _extended_gcd {
-  my ($a, $b) = @_;
-  my $zero = $a-$a;
-  my ($x, $lastx, $y, $lasty) = ($zero, $zero+1, $zero+1, $zero);
-  while ($b != 0) {
-    my $q = int($a/$b);
-    ($a, $b) = ($b, $a % $b);
-    ($x, $lastx) = ($lastx - $q*$x, $x);
-    ($y, $lasty) = ($lasty - $q*$y, $y);
-  }
-  return ($a, $lastx, $lasty);
-}
+#sub _extended_gcd {
+#  my ($a, $b) = @_;
+#  my $zero = $a-$a;
+#  my ($x, $lastx, $y, $lasty) = ($zero, $zero+1, $zero+1, $zero);
+#  while ($b != 0) {
+#    my $q = int($a/$b);
+#    ($a, $b) = ($b, $a % $b);
+#    ($x, $lastx) = ($lastx - $q*$x, $x);
+#    ($y, $lasty) = ($lasty - $q*$y, $y);
+#  }
+#  return ($a, $lastx, $lasty);
+#}
 
 sub normalize {
   my ($self) = @_;
@@ -199,6 +199,8 @@ __END__
 
 =for stopwords mul
 
+=for test_synopsis use v5.14;  my($c,$n,$k,$ECP2);
+
 
 =head1 NAME
 
@@ -207,7 +209,7 @@ Math::Prime::Util::ECProjectivePoint - Elliptic curve operations for projective
 
 =head1 VERSION
 
-Version 0.42
+Version 0.46
 
 
 =head1 SYNOPSIS
@@ -215,11 +217,11 @@ Version 0.42
   # Create a point on a curve (a,b,n) with coordinates 0,1
   my $ECP = Math::Prime::Util::ECProjectivePoint->new($c, $n, 0, 1);
 
-  # scalar multiplication by k.
-  $ECP->mul($k)
+  # scalar multiplication by $k.
+  $ECP->mul($k);
 
   # add two points on the same curve
-  $ECP->add($ECP2)
+  $ECP->add($ECP2);
 
   say "P = O" if $ECP->is_infinity();
 
