@@ -18,7 +18,7 @@ static const UV mr_bases_const2[1] = {2};
   Code inside USE_MONT_PRIMALITY is Montgomery math and efficient M-R from
   Wojciech Izykowski.  See:  https://github.com/wizykowski/miller-rabin
 
-Copyright (c) 2013, Wojciech Izykowski
+Copyright (c) 2013-2014, Wojciech Izykowski
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,19 +43,24 @@ ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
 #if USE_MONT_PRIMALITY
+
+#if defined(__GNUC__) && (__GNUC__ >= 3)
+ #define MPU_UNLIKELY(x) __builtin_expect(!!(x), 0)
+#else
+ #define MPU_UNLIKELY(x) (x)
+#endif
+
 static INLINE uint64_t mont_prod64(uint64_t a, uint64_t b, uint64_t n, uint64_t npi)
 {
   uint64_t t_hi, t_lo, m, mn_hi, mn_lo, u;
-  int carry;
   /* t_hi * 2^64 + t_lo = a*b */
   asm("mulq %3" : "=a"(t_lo), "=d"(t_hi) : "a"(a), "rm"(b));
+  if (MPU_UNLIKELY(t_lo == 0)) return t_hi;
   m = t_lo * npi;
   /* mn_hi * 2^64 + mn_lo = m*n */
   asm("mulq %3" : "=a"(mn_lo), "=d"(mn_hi) : "a"(m), "rm"(n));
-  carry = t_lo + mn_lo < t_lo ? 1 : 0;
-  u = t_hi + mn_hi + carry;
-  if (u < t_hi) return u-n;
-  return u >= n ? u-n : u;
+  u = t_hi + mn_hi + 1;
+  return (u < t_hi || u >= n)  ?  u-n  :  u;
 }
 #define mont_square64(a, n, npi)  mont_prod64(a, a, n, npi)
 static INLINE UV mont_powmod64(uint64_t a, uint64_t k, uint64_t one, uint64_t n, uint64_t npi)
@@ -68,28 +73,31 @@ static INLINE UV mont_powmod64(uint64_t a, uint64_t k, uint64_t one, uint64_t n,
   }
   return t;
 }
+/* Returns -a^-1 mod 2^64.  From B. Arazi "On Primality Testing Using Purely
+ * Divisionless Operations", Computer Journal (1994) 37 (3): 219-222, Proc 5 */
 static INLINE uint64_t modular_inverse64(const uint64_t a)
 {
-  uint64_t u,x,w,z,q;
+  uint64_t S = 1, J = 0;
+  int idx;
+  /* Basic algorithm:
+   *    for (i = 0; i < 64; i++) {
+   *      if (S & 1)  {  J |= (1ULL << i);  S += a;  }
+   *      S >>= 1;
+   *    }
+   * What follows is 8 bits at a time, unrolled by hand. */
+  static const char mask[128] = {255,85,51,73,199,93,59,17,15,229,195,89,215,237,203,33,31,117,83,105,231,125,91,49,47,5,227,121,247,13,235,65,63,149,115,137,7,157,123,81,79,37,3,153,23,45,11,97,95,181,147,169,39,189,155,113,111,69,35,185,55,77,43,129,127,213,179,201,71,221,187,145,143,101,67,217,87,109,75,161,159,245,211,233,103,253,219,177,175,133,99,249,119,141,107,193,191,21,243,9,135,29,251,209,207,165,131,25,151,173,139,225,223,53,19,41,167,61,27,241,239,197,163,57,183,205,171,1};
 
-  x = 1; z = a;
-
-  q = (-z)/z + 1;  /* = 2^64 / z                 */
-  u = - q;         /* = -q * x                   */
-  w = - q * z;     /* = b - q * z = 2^64 - q * z */
-
-  /* after first iteration all variables are 64-bit */
-
-  while (w) {
-    if (w < z) {
-      q = u; u = x; x = q; /* swap(u, x) */
-      q = w; w = z; z = q; /* swap(w, z) */
-    }
-    q = w / z;
-    u -= q * x;
-    w -= q * z;
-  }
-  return x;
+  const char amask = mask[(a >> 1) & 127];
+  uint32_t T;
+  idx = (amask*(S&255)) & 255;  J = idx;                  S = (S+a*idx) >> 8;
+  idx = (amask*(S&255)) & 255;  J |= (uint64_t)idx << 8;  S = (S+a*idx) >> 8;
+  idx = (amask*(S&255)) & 255;  J |= (uint64_t)idx <<16;  S = (S+a*idx) >> 8;
+  idx = (amask*(S&255)) & 255;  J |= (uint64_t)idx <<24;  T = (S+a*idx) >> 8;
+  idx = (amask*(T&255)) & 255;  J |= (uint64_t)idx <<32;  T = (T+a*idx) >> 8;
+  idx = (amask*(T&255)) & 255;  J |= (uint64_t)idx <<40;  T = (T+a*idx) >> 8;
+  idx = (amask*(T&255)) & 255;  J |= (uint64_t)idx <<48;  T = (T+a*idx) >> 8;
+  idx = (amask*(T&255)) & 255;  J |= (uint64_t)idx <<56;
+  return J;
 }
 static INLINE uint64_t compute_modn64(const uint64_t n)
 {
@@ -118,7 +126,7 @@ static INLINE uint64_t compute_2_65_mod_n(const uint64_t n, const uint64_t modn)
 static int monty_mr64(const uint64_t n, const UV* bases, int cnt)
 {
   int i, j, t;
-  const uint64_t npi = modular_inverse64(-((int64_t)n));
+  const uint64_t npi = modular_inverse64(n);
   const uint64_t r = compute_modn64(n);
   uint64_t u = n - 1;
   const uint64_t nr = n - r;
@@ -166,7 +174,21 @@ static int jacobi_iu(IV in, UV m) {
   return (m == 1) ? j : 0;
 }
 
-
+static UV select_extra_strong_parameters(UV n, UV increment) {
+  UV P = 3;
+  while (1) {
+    UV D = P*P - 4;
+    if (gcd_ui(D, n) > 1 && gcd_ui(D, n) != n) return 0;
+    if (jacobi_iu(D, n) == -1)
+      break;
+    if (P == (3+20*increment) && is_perfect_square(n)) return 0;
+    P += increment;
+    if (P > 65535)
+      croak("lucas_extrastrong_params: P exceeded 65535");
+  }
+  if (P >= n)  P %= n;   /* Never happens with increment < 4 */
+  return P;
+}
 
 
 /* Fermat pseudoprime */
@@ -191,19 +213,18 @@ int _XS_is_pseudoprime(UV const n, UV a)
  */
 int _XS_miller_rabin(UV const n, const UV *bases, int nbases)
 {
+#if USE_MONT_PRIMALITY
+  MPUassert(n > 3, "MR called with n <= 3");
+  if ((n & 1) == 0) return 0;
+  return monty_mr64((uint64_t)n, bases, nbases);
+#else
   UV d = n-1;
   int b, r, s = 0;
 
   MPUassert(n > 3, "MR called with n <= 3");
   if ((n & 1) == 0) return 0;
 
-#if USE_MONT_PRIMALITY
-  if (n >= UVCONST(4294967295))
-    return monty_mr64((uint64_t)n, bases, nbases);
-#endif
-
   while (!(d&1)) {  s++;  d >>= 1;  }
-
   for (b = 0; b < nbases; b++) {
     UV x, a = bases[b];
     if (a < 2)  croak("Base %"UVuf" is invalid", a);
@@ -224,6 +245,7 @@ int _XS_miller_rabin(UV const n, const UV *bases, int nbases)
     if (r >= s)  return 0;
   }
   return 1;
+#endif
 }
 
 int _XS_BPSW(UV const n)
@@ -235,11 +257,8 @@ int _XS_BPSW(UV const n)
   return    _XS_miller_rabin(n, mr_bases_const2, 1)
          && _XS_is_almost_extra_strong_lucas_pseudoprime(n,1);
 #else
-  if (n < UVCONST(4294967295)) {
-    return    _XS_miller_rabin(n, mr_bases_const2, 1)
-           && _XS_is_almost_extra_strong_lucas_pseudoprime(n,1);
-  } else {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t montr = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, montr);
     uint64_t u = n-1;
@@ -265,19 +284,8 @@ int _XS_BPSW(UV const n)
       }
     }
     /* AES Lucas test */
-    P = 3;
-    while (1) {
-      UV D = P*P - 4;
-      d = gcd_ui(D, n);
-      if (d > 1 && d < n)
-        return 0;
-      if (jacobi_iu(D, n) == -1)
-        break;
-      if (P == (3+20) && is_perfect_square(n)) return 0;
-      P++;
-      if (P > 65535)
-        croak("lucas_extrastrong_params: P exceeded 65535");
-    }
+    P = select_extra_strong_parameters(n, 1);
+    if (P == 0) return 0;
 
     d = n+1;
     s = 0;
@@ -320,6 +328,7 @@ void lucas_seq(UV* Uret, UV* Vret, UV* Qkret, UV n, IV P, IV Q, UV k)
 {
   UV U, V, b, Dmod, Qmod, Pmod, Qk;
 
+  MPUassert(n > 1, "lucas_sequence:  modulus n must be > 1");
   if (k == 0) {
     *Uret = 0;
     *Vret = 2;
@@ -327,10 +336,16 @@ void lucas_seq(UV* Uret, UV* Vret, UV* Qkret, UV n, IV P, IV Q, UV k)
     return;
   }
 
-  Qmod = (Q < 0)  ?  (UV)(Q + (IV)n)  :  (UV)Q;
-  Pmod = (P < 0)  ?  (UV)(P + (IV)n)  :  (UV)P;
+  Qmod = (Q < 0)  ?  (UV) (Q + (IV)(((-Q/n)+1)*n))  :  (UV)Q;
+  Pmod = (P < 0)  ?  (UV) (P + (IV)(((-P/n)+1)*n))  :  (UV)P;
   Dmod = submod( mulmod(Pmod, Pmod, n), mulmod(4, Qmod, n), n);
-  MPUassert(Dmod != 0, "lucas_seq: D is 0");
+  if (Dmod == 0) {
+    b = Pmod >> 1;
+    *Uret = mulmod(k, powmod(b, k-1, n), n);
+    *Vret = mulmod(2, powmod(b, k, n), n);
+    *Qkret = powmod(Qmod, k, n);
+    return;
+  }
   U = 1;
   V = Pmod;
   Qk = Qmod;
@@ -418,16 +433,10 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     P = 1;
     Q = (1 - D) / 4;
   } else {
-    P = 3;
+    P = select_extra_strong_parameters(n, 1);
+    if (P == 0) return 0;
     Q = 1;
-    while (1) {
-      D = P*P - 4;
-      if (gcd_ui(D, n) > 1 && gcd_ui(D, n) != n) return 0;
-      if (jacobi_iu(D, n) == -1)
-        break;
-      if (P == 21 && is_perfect_square(n)) return 0;
-      P++;
-    }
+    D = P*P - 4;
   }
   MPUassert( D == (P*P - 4*Q) , "is_lucas_pseudoprime: incorrect DPQ");
 
@@ -437,8 +446,8 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     while ( (d & 1) == 0 ) {  s++;  d >>= 1; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t mont1 = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, mont1);
     const uint64_t montP = (P == 1) ? mont1
@@ -520,8 +529,7 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     }
     return 0;
   }
-#endif
-
+#else
   lucas_seq(&U, &V, &Qk, n, P, Q, d);
 
   if (strength == 0) {
@@ -552,14 +560,17 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     }
   }
   return 0;
+#endif
 }
 
 /* A generalization of Pari's shortcut to the extra-strong Lucas test.
+ *
+ * This only calculates and tests V, which means less work, but it does result
+ * in a few more pseudoprimes than the full extra-strong test.
+ *
  * I've added a gcd check at the top, which needs to be done and also results
  * in fewer pseudoprimes.  Pari always does trial division to 100 first so
- * is unlikely to come up there.  This only calculate V, which can be done
- * faster, but that means we have more pseudoprimes than the standard
- * extra-strong test.
+ * is unlikely to come up there.
  *
  * increment:  1 for Baillie OEIS, 2 for Pari.
  *
@@ -575,20 +586,8 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
   if (increment < 1 || increment > 256)
     croak("Invalid lucas parameter increment: %"UVuf"\n", increment);
 
-  P = 3;
-  while (1) {
-    UV D = P*P - 4;
-    d = gcd_ui(D, n);
-    if (d > 1 && d < n)
-      return 0;
-    if (jacobi_iu(D, n) == -1)
-      break;
-    if (P == (3+20*increment) && is_perfect_square(n)) return 0;
-    P += increment;
-    if (P > 65535)
-      croak("lucas_extrastrong_params: P exceeded 65535");
-  }
-  if (P >= n)  P %= n;   /* Never happens with increment < 4 */
+  P = select_extra_strong_parameters(n, increment);
+  if (P == 0) return 0;
 
   d = n+1;
   s = 0;
@@ -596,8 +595,8 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
   { UV v = d; b = 0; while (v >>= 1) b++; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t montr = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, montr);
     const uint64_t montP = compute_a_times_2_64_mod_n(P, n, montr);
@@ -625,7 +624,7 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
     }
     return 0;
   }
-#endif
+#else
   W = mulsubmod(P, P, 2, n);
   V = P;
   while (b--) {
@@ -648,6 +647,95 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
       return 0;
   }
   return 0;
+#endif
+}
+
+static void mat_mulmod_3x3(UV* a, UV* b, UV n) {
+  int i, row, col;
+  UV i1, i2, i3, t[9];
+  for (row = 0; row < 3; row++) {
+    for (col = 0; col < 3; col++) {
+      if (n < HALF_WORD/2) {
+        i1 = a[3*row+0] * b[0+col];
+        i2 = a[3*row+1] * b[3+col];
+        i3 = a[3*row+2] * b[6+col];
+        t[3*row+col] = (i1 + i2 + i3) % n;
+      } else {
+        i1 = mulmod(a[3*row+0], b[0+col], n);
+        i2 = mulmod(a[3*row+1], b[3+col], n);
+        i3 = mulmod(a[3*row+2], b[6+col], n);
+        t[3*row+col] = addmod( addmod(i1, i2, n), i3, n );
+      }
+    }
+  }
+  for (i = 0; i < 9; i++) a[i] = t[i];
+}
+static void mat_powmod_3x3(UV* m, UV k, UV n) {
+  UV res[9] = {1,0,0, 0,1,0, 0,0,1};
+  int i;
+
+  while (k) {
+    if (k & 1)  mat_mulmod_3x3(res, m, n);
+    k >>= 1;
+    if (k)      mat_mulmod_3x3(m, m, n);
+  }
+  for (i = 0; i < 9; i++)  m[i] = res[i];
+}
+
+int is_perrin_pseudoprime(UV n)
+{
+  UV m[9] = {0,1,0, 0,0,1, 1,1,0};
+  mat_powmod_3x3(m, n, n);
+  /* P(n) = sum of diagonal  =  3*top-left + 2*top-right */
+  return (addmod( addmod(m[0], m[4], n), m[8], n) == 0);
+}
+
+int is_frobenius_pseudoprime(UV n, IV P, IV Q)
+{
+  UV U, V, Qk, Vcomp;
+  int k = 0;
+  IV D;
+  UV Du, Pu, Qu;
+
+  if (n < 7) return (n == 2 || n == 3 || n == 5);
+  if ((n % 2) == 0 || n == UV_MAX) return 0;
+  if (is_perfect_square(n)) return 0;
+
+  if (P == 0 && Q == 0) {
+    P = -1; Q = 2;
+    while (k != -1) {
+      P += 2;
+      if (P == 3) P = 5;  /* P=3,Q=2 -> D=9-8=1 => k=1, so skip */
+      D = P*P-4*Q;
+      Du = D >= 0 ? D : -D;
+      if ((UV)P >= n || Du >= n) break;
+      k = kronecker_su(D, n);
+      if (k == 0) return 0;
+    }
+    if (_XS_get_verbose()) printf("%"UVuf" Frobenius (%"IVdf",%"IVdf") : x^2 - %"IVdf"x + %"IVdf"\n", n, P, Q, P, Q);
+    Vcomp = 4;
+  } else {
+    D = P*P-4*Q;
+    Du = D >= 0 ? D : -D;
+    if (D != 5 && is_perfect_square(Du))
+      croak("Frobenius invalid P,Q: (%"IVdf",%"IVdf")", P, Q);
+  }
+  Pu = P >= 0 ? P : -P;
+  Qu = Q >= 0 ? Q : -Q;
+
+  if (n <= Du || n <= Qu || n <= Pu) return is_prob_prime(n);
+  if (gcd_ui(n, Pu*Qu*Du) != 1) return 0;
+  if (k == 0) {
+    k = kronecker_su(D, n);
+    if (k == 0) return 0;
+    Qu = addmod(Qu,Qu,n);
+    Vcomp = (k == 1)  ?  2  :  Q >= 0 ? Qu : n-Qu;
+  }
+
+  lucas_seq(&U, &V, &Qk, n, P, Q, n-k);
+  /* if (_XS_get_verbose()) printf("%"UVuf" Frobenius U = %"UVuf" V = %"UVuf"\n", n, U, V); */
+  if (U == 0 && V == Vcomp) return 1;
+  return 0;
 }
 
 /*
@@ -681,8 +769,8 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
   { UV v = np1; len = 1;  while (v >>= 1) len++; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t mont1 = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, mont1);
     const uint64_t mont5 = compute_a_times_2_64_mod_n(5, n, mont1);
@@ -719,7 +807,7 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
     }
     return (a == 0 && b == result);
   }
-#endif
+#else
   a = 1;
   b = 2;
 
@@ -754,6 +842,7 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
   if (a == 0 && b == result)
     return 1;
   return 0;
+#endif
 }
 
 
@@ -778,6 +867,37 @@ int is_prob_prime(UV n)
 {
   int ret;
 
+#if !USE_MONT_PRIMALITY
+  /* On some platforms, especially where UV is a long long, trial division
+   * for small inputs can be much faster.  The exact crossover will differ. */
+  if (n < 20000000) {
+    unsigned int m = n;
+    if (m < 11) {
+      if (m == 2 || m == 3 || m == 5 || m == 7)     return 2;
+      else                                          return 0;
+    }
+    if (!(m%2) || !(m%3) || !(m%5) || !(m%7))       return 0;
+    if (m >= 121) {
+      unsigned int f = 11;
+      unsigned int limit = isqrt(n);
+      while (f <= limit) {
+        if ((m%f) == 0)  return 0;  f += 2;
+        if ((m%f) == 0)  return 0;  f += 4;
+        if ((m%f) == 0)  return 0;  f += 2;
+        if ((m%f) == 0)  return 0;  f += 4;
+        if ((m%f) == 0)  return 0;  f += 6;
+        if ((m%f) == 0)  return 0;  f += 2;
+        if ((m%f) == 0)  return 0;  f += 6;
+        if ((m%f) == 0)  return 0;  f += 4;
+      }
+    }
+    return 2;
+  }
+  if (!(n%2) || !(n%3) || !(n%5) || !(n%7))       return 0;
+  if (!(n%11) || !(n%13) || !(n%17) || !(n%19) ||
+      !(n%23) || !(n%29) || !(n%31) || !(n%37) ||
+      !(n%41) || !(n%43) || !(n%47) || !(n%53))   return 0;
+#else
   if (n < 11) {
     if (n == 2 || n == 3 || n == 5 || n == 7)     return 2;
     else                                          return 0;
@@ -788,6 +908,7 @@ int is_prob_prime(UV n)
       !(n%23) || !(n%29) || !(n%31) || !(n%37) ||
       !(n%41) || !(n%43) || !(n%47) || !(n%53))   return 0;
   if (n < 3481) /* 59*59 */                       return 2;
+#endif
 
 #if BITS_PER_WORD == 32
   /* We could use one base when n < 49191, two when n < 360018361. */
